@@ -31,14 +31,14 @@ int main(argc, argv)
   printf("Searching for the number = %d...\n", pid);
 
   /* Initialize the mutex lock */
-  pthread_mutex_init(&lock, NULL);
+  pthread_mutex_init(&lock, NULL); 
 
   /* Create the searching threads */
   for (i=0; i<NUM_THREADS; i++)
     pthread_create(&threads[i], NULL, search, (void *)pid);
 
   /* Wait for (join) all the searching threads */
-  for (i=0; i<NUM_THREADS; i++)
+  for (i=0; i<NUM_THREADS; i++) 
     pthread_join(threads[i], NULL);
 
   printf("It took %d tries to find the number.\n", tries);
@@ -47,7 +47,7 @@ int main(argc, argv)
   return 0;
 }
 
-/* This is the cleanup function that is called
+/* This is the cleanup function that is called 
    when the threads are cancelled */
 
 void print_it(void *arg)
@@ -59,7 +59,7 @@ void print_it(void *arg)
   tid = pthread_self();
 
   /* Print where the thread was in its search when it was cancelled */
-  printf("Thread %lx was canceled on its %d try.\n", tid, *try);
+  printf("Thread %lx was canceled on its %d try.\n", tid, *try); 
 }
 
 /* This is the search routine that is executed in each thread */
@@ -74,25 +74,28 @@ void *search(void *arg)
   tid = pthread_self();
 
   /* use the thread ID to set the seed for the random number generator */
+  /* Since srand and rand are not thread-safe, serialize with lock */
+  pthread_mutex_lock(&lock);
   srand((int)tid);
   i = rand() & 0xFFFFFF;
+  pthread_mutex_unlock(&lock);
   ntries = 0;
 
   /* Set the cancellation parameters --
-     - Enable thread cancellation
+     - Enable thread cancellation 
      - Defer the action of the cancellation */
 
   pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, NULL);
   pthread_setcanceltype(PTHREAD_CANCEL_DEFERRED, NULL);
 
   /* Push the cleanup routine (print_it) onto the thread
-     cleanup stack.  This routine will be called when the
+     cleanup stack.  This routine will be called when the 
      thread is cancelled.  Also note that the pthread_cleanup_push
      call must have a matching pthread_cleanup_pop call.  The
-     push and pop calls MUST be at the same lexical level
+     push and pop calls MUST be at the same lexical level 
      within the code */
 
-  /* Pass address of `ntries' since the current value of `ntries' is not
+  /* Pass address of `ntries' since the current value of `ntries' is not 
      the one we want to use in the cleanup function */
 
   pthread_cleanup_push(print_it, (void *)&ntries);
@@ -115,20 +118,16 @@ void *search(void *arg)
       printf("Thread %lx found the number!\n", tid);
 
       /* Cancel all the other threads */
-      for (j=0; j<NUM_THREADS; j++)
+      for (j=0; j<NUM_THREADS; j++) 
         if (threads[j] != tid) pthread_cancel(threads[j]);
 
       /* Break out of the while loop */
       break;
     }
 
-    /* Every 100 tries check to see if the thread has been cancelled.
-       If the thread has not been cancelled then yield the processor
-       LWP to another thread that may be able to run. */
-
+    /* Every 100 tries check to see if the thread has been cancelled. */
     if (ntries % 100 == 0) {
       pthread_testcancel();
-      sched_yield();
     }
   }
 
@@ -142,3 +141,4 @@ void *search(void *arg)
   pthread_cleanup_pop(0);
   return((void *)0);
 }
+
