@@ -21,12 +21,12 @@
 
 int * __errno_location()
 {
-  pthread_t self = thread_self();
+  pthread_descr self = thread_self();
   return &(self->p_errno);
 }
 
 int * __h_errno_location()
 {
-  pthread_t self = thread_self();
+  pthread_descr self = thread_self();
   return &(self->p_h_errno);
 }
