@@ -16,19 +16,53 @@
 
 /* Includes */
 
+#include <libc-lock.h>
 #include <setjmp.h>
 #include <unistd.h>
 #include <sys/types.h>
 
 #include "pt-machine.h"
 
+/* Arguments passed to thread creation routine */
+
+struct pthread_start_args {
+  void * (*start_routine)(void *); /* function to run */
+  void * arg;                   /* its argument */
+  sigset_t mask;                /* initial signal mask for thread */
+  int schedpolicy;              /* initial scheduling policy (if any) */
+  struct sched_param schedparam; /* initial scheduling parameters (if any) */
+};
+
+
+/* We keep thread specific data in a special data structure, a two-level
+   array.  The top-level array contains pointers to dynamically allocated
+   arrays of a certain number of data pointers.  So we can implement a
+   sparse array.  Each dynamic second-level array has
+       PTHREAD_KEY_2NDLEVEL_SIZE
+   entries.  This value shouldn't be too large.  */
+#define PTHREAD_KEY_2NDLEVEL_SIZE      32
+
+/* We need to address PTHREAD_KEYS_MAX key with PTHREAD_KEY_2NDLEVEL_SIZE
+   keys in each subarray.  */
+#define PTHREAD_KEY_1STLEVEL_SIZE \
+  ((PTHREAD_KEYS_MAX + PTHREAD_KEY_2NDLEVEL_SIZE - 1) \
+   / PTHREAD_KEY_2NDLEVEL_SIZE)
+
+
+#define PTHREAD_START_ARGS_INITIALIZER { NULL, NULL, {{0, }}, 0, { 0 } }
+
 /* The type of thread descriptors */
 
-struct _pthread {
-  pthread_t p_nextlive, p_prevlive; /* Double chaining of active threads */
-  pthread_t p_nextwaiting;      /* Next element in the queue holding the thr */
+typedef struct _pthread_descr_struct * pthread_descr;
+
+struct _pthread_descr_struct {
+  pthread_descr p_nextlive, p_prevlive;
+                                /* Double chaining of active threads */
+  pthread_descr p_nextwaiting;  /* Next element in the queue holding the thr */
+  pthread_t p_tid;              /* Thread identifier */
   int p_pid;                    /* PID of Unix process */
-  int p_spinlock;               /* Spinlock for synchronized accesses */
+  int p_priority;               /* Thread priority (== 0 if not realtime) */
+  int * p_spinlock;             /* Spinlock for synchronized accesses */
   int p_signal;                 /* last signal received */
   sigjmp_buf * p_signal_jmp;    /* where to siglongjmp on a signal or NULL */
   sigjmp_buf * p_cancel_jmp;    /* where to siglongjmp on a cancel or NULL */
@@ -37,23 +71,31 @@ struct _pthread {
   char p_exited;                /* true if the assoc. process terminated */
   void * p_retval;              /* placeholder for return value */
   int p_retcode;                /* placeholder for return code */
-  pthread_t p_joining;          /* thread joining on that thread or NULL */
+  pthread_descr p_joining;      /* thread joining on that thread or NULL */
   struct _pthread_cleanup_buffer * p_cleanup; /* cleanup functions */
   char p_cancelstate;           /* cancellation state */
   char p_canceltype;            /* cancellation type (deferred/async) */
   char p_canceled;              /* cancellation request pending */
   int p_errno;                  /* error returned by last system call */
   int p_h_errno;                /* error returned by last netdb function */
-  void *(*p_initial_fn)(void *); /* function to call on thread start */
-  void *p_initial_fn_arg;	/* argument to give that function */
-  sigset_t p_initial_mask;	/* signal mask on thread start */
-  void * p_specific[PTHREAD_KEYS_MAX]; /* thread-specific data */
+  struct pthread_start_args p_start_args; /* arguments for thread creation */
+  void ** p_specific[PTHREAD_KEY_1STLEVEL_SIZE]; /* thread-specific data */
+  void * p_libc_specific[_LIBC_TSD_KEY_N]; /* thread-specific data for libc */
+};
+
+/* The type of thread handles. */
+
+typedef struct pthread_handle_struct * pthread_handle;
+
+struct pthread_handle_struct {
+  int h_spinlock;               /* Spinlock for sychronized access */
+  pthread_descr h_descr;        /* Thread descriptor or NULL if invalid */
 };
 
 /* The type of messages sent to the thread manager thread */
 
 struct pthread_request {
-  pthread_t req_thread;         /* Thread doing the request */
+  pthread_descr req_thread;     /* Thread doing the request */
   enum {                        /* Request kind */
     REQ_CREATE, REQ_FREE, REQ_PROCESS_EXIT, REQ_MAIN_THREAD_EXIT
   } req_kind;
@@ -65,7 +107,7 @@ struct pthread_request {
       sigset_t mask;            /*   signal mask */
     } create;
     struct {                    /* For REQ_FREE: */
-      pthread_t thread;         /*   ID of thread to free */
+      pthread_descr thread;     /*   descriptor of thread to free */
     } free;
     struct {                    /* For REQ_PROCESS_EXIT: */
       int code;                 /*   exit status */
@@ -80,17 +122,23 @@ struct pthread_request {
 #define PTHREAD_SIG_RESTART SIGUSR1
 #define PTHREAD_SIG_CANCEL SIGUSR2
 
+/* Global array of thread handles, used for validating a thread id
+   and retrieving the corresponding thread descriptor. Also used for
+   mapping the available stack segments. */
+
+extern struct pthread_handle_struct __pthread_handles[PTHREAD_THREADS_MAX];
+
 /* Descriptor of the initial thread */
 
-extern struct _pthread __pthread_initial_thread;
+extern struct _pthread_descr_struct __pthread_initial_thread;
 
 /* Descriptor of the manager thread */
 
-extern struct _pthread __pthread_manager_thread;
+extern struct _pthread_descr_struct __pthread_manager_thread;
 
 /* Descriptor of the main thread */
 
-extern pthread_t __pthread_main_thread;
+extern pthread_descr __pthread_main_thread;
 
 /* Limit between the stack of the initial thread (above) and the
    stacks of other threads (below). Aligned on a STACK_SIZE boundary.
@@ -106,7 +154,7 @@ extern int __pthread_manager_request;
 
 /* Other end of the pipe for sending requests to the thread manager. */
 
-extern int pthread_manager_reader;
+extern int __pthread_manager_reader;
 
 /* Limits of the thread manager stack. */
 
@@ -117,6 +165,20 @@ extern char *__pthread_manager_thread_tos;
 
 extern int __pthread_exit_requested, __pthread_exit_code;
 
+/* Return the handle corresponding to a thread id */
+
+static inline pthread_handle thread_handle(pthread_t id)
+{
+  return &__pthread_handles[id % PTHREAD_THREADS_MAX];
+}
+
+/* Validate a thread handle. Must have acquired h->h_spinlock before. */
+
+static inline int invalid_handle(pthread_handle h, pthread_t id)
+{
+  return h->h_descr == NULL || h->h_descr->p_tid != id;
+}
+
 /* Fill in defaults left unspecified by pt-machine.h.  */
 
 /* The page size we can get from the system.  This should likely not be
@@ -137,7 +199,8 @@ extern int __pthread_exit_requested, __pthread_exit_code;
 #define INITIAL_STACK_SIZE  (4 * PAGE_SIZE)
 #endif
 
-/* Size of the thread manager stack */
+/* Size of the thread manager stack. The "- 32" avoids wasting space
+   with some malloc() implementations. */
 #ifndef THREAD_MANAGER_STACK_SIZE
 #define THREAD_MANAGER_STACK_SIZE  (2 * PAGE_SIZE - 32)
 #endif
@@ -157,8 +220,8 @@ extern int __pthread_exit_requested, __pthread_exit_code;
 
 /* Recover thread descriptor for the current thread */
 
-static inline pthread_t thread_self (void) __attribute__ ((const));
-static inline pthread_t thread_self (void)
+static inline pthread_descr thread_self (void) __attribute__ ((const));
+static inline pthread_descr thread_self (void)
 {
 #ifdef THREAD_SELF
   THREAD_SELF
@@ -170,7 +233,7 @@ static inline pthread_t thread_self (void)
 	   && sp < __pthread_manager_thread_tos)
     return &__pthread_manager_thread;
   else
-    return (pthread_t) (((unsigned long int) sp | (STACK_SIZE - 1)) + 1) - 1;
+    return (pthread_descr)(((unsigned long)sp | (STACK_SIZE-1))+1) - 1;
 #endif
 }
 
@@ -179,7 +242,7 @@ static inline pthread_t thread_self (void)
 #ifdef DEBUG
 #include <assert.h>
 #define ASSERT assert
-#define MSG pthread_message
+#define MSG __pthread_message
 #else
 #define ASSERT(x)
 #define MSG(msg,arg...)
@@ -191,7 +254,7 @@ void __pthread_destroy_specifics(void);
 void __pthread_perform_cleanup(void);
 void __pthread_sighandler(int sig);
 void __pthread_message(char * fmt, long arg, ...);
-int __pthread_manager(void *arg);
+int __pthread_manager(void *reqfd);
 void __pthread_manager_sighandler(int sig);
 void __pthread_reset_main_thread(void);
 void __fresetlockfiles(void);
