@@ -75,7 +75,7 @@ static void pthread_kill_all_threads(int sig, int main_thread_also);
 
 int __pthread_manager(void *arg)
 {
-  int reqfd = (long)arg;
+  int reqfd = (int)arg;
   sigset_t mask;
   fd_set readfds;
   struct timeval timeout;
@@ -100,7 +100,7 @@ int __pthread_manager(void *arg)
     /* Check for termination of the main thread */
     if (getppid() == 1) {
       pthread_kill_all_threads(SIGKILL, 0);
-      return 0;
+      _exit(0);
     }
     /* Check for dead children */
     if (terminated_children) {
@@ -153,7 +153,7 @@ static int pthread_start_thread(void *arg)
 #endif
   /* Make sure our pid field is initialized, just in case we get there
      before our father has initialized it. */
-  self->p_pid = getpid();
+  self->p_pid = __getpid();
   /* Initial signal mask is that of the creating thread. (Otherwise,
      we'd just inherit the mask of the thread manager.) */
   sigprocmask(SIG_SETMASK, &self->p_start_args.mask, NULL);
@@ -172,7 +172,7 @@ static int pthread_handle_create(pthread_t *thread, const pthread_attr_t *attr,
                                  void * (*start_routine)(void *), void *arg,
                                  sigset_t * mask, int father_pid)
 {
-  int sseg;
+  size_t sseg;
   int pid;
   pthread_descr new_thread;
   pthread_t new_thread_id;
@@ -212,7 +212,8 @@ static int pthread_handle_create(pthread_t *thread, const pthread_attr_t *attr,
   new_thread->p_canceled = 0;
   new_thread->p_errno = 0;
   new_thread->p_h_errno = 0;
-  for (i = 0; i < PTHREAD_KEYS_MAX; i++) new_thread->p_specific[i] = NULL;
+  for (i = 0; i < PTHREAD_KEY_1STLEVEL_SIZE; i++)
+    new_thread->p_specific[i] = NULL;
   /* Initialize the thread handle */
   __pthread_handles[sseg].h_spinlock = 0; /* should already be 0 */
   __pthread_handles[sseg].h_descr = new_thread;
@@ -247,7 +248,7 @@ static int pthread_handle_create(pthread_t *thread, const pthread_attr_t *attr,
     munmap((caddr_t)((char *)(new_thread+1) - INITIAL_STACK_SIZE),
            INITIAL_STACK_SIZE);
     __pthread_handles[sseg].h_descr = NULL;
-    return EAGAIN;
+    return errno;
   }
   /* Insert new thread in doubly linked list of active threads */
   new_thread->p_prevlive = __pthread_main_thread;
