@@ -69,7 +69,17 @@ int __pthread_mutex_trylock(pthread_mutex_t * mutex)
       return 0;
     }
     break;
+  case PTHREAD_MUTEX_ERRORCHECK_NP:
+    self = thread_self();
+    if (mutex->m_count == 0) {
+      mutex->m_count = 1;
+      mutex->m_owner = self;
+      release(&mutex->m_spinlock);
+      return 0;
+    }
+    break;
   default:
+    release(&mutex->m_spinlock);
     return EINVAL;
   }
   release(&mutex->m_spinlock);
@@ -101,7 +111,20 @@ int __pthread_mutex_lock(pthread_mutex_t * mutex)
         return 0;
       }
       break;
+    case PTHREAD_MUTEX_ERRORCHECK_NP:
+      self = thread_self();
+      if (mutex->m_count == 0) {
+       mutex->m_count = 1;
+       mutex->m_owner = self;
+       release(&mutex->m_spinlock);
+       return 0;
+      } else if (mutex->m_owner == self) {
+       release(&mutex->m_spinlock);
+       return EDEADLK;
+      }
+      break;
     default:
+      release(&mutex->m_spinlock);
       return EINVAL;
     }
     /* Suspend ourselves, then try again */
@@ -129,7 +152,15 @@ int __pthread_mutex_unlock(pthread_mutex_t * mutex)
     }
     mutex->m_count = 0; /* so that excess unlocks do not break everything */
     break;
+  case PTHREAD_MUTEX_ERRORCHECK_NP:
+    if (mutex->m_count == 0 || mutex->m_owner != thread_self()) {
+      release(&mutex->m_spinlock);
+      return EPERM;
+    }
+    mutex->m_count = 0;
+    break;
   default:
+    release(&mutex->m_spinlock);
     return EINVAL;
   }
   th = dequeue(&mutex->m_waiting);
@@ -154,7 +185,9 @@ weak_alias (__pthread_mutexattr_destroy, pthread_mutexattr_destroy)
 
 int __pthread_mutexattr_setkind_np(pthread_mutexattr_t *attr, int kind)
 {
-  if (kind != PTHREAD_MUTEX_FAST_NP && kind != PTHREAD_MUTEX_RECURSIVE_NP)
+  if (kind != PTHREAD_MUTEX_FAST_NP
+      && kind != PTHREAD_MUTEX_RECURSIVE_NP
+      && kind != PTHREAD_MUTEX_ERRORCHECK_NP)
     return EINVAL;
   attr->mutexkind = kind;
   return 0;
