@@ -32,7 +32,7 @@ struct _pthread_descr_struct __pthread_initial_thread = {
   &__pthread_initial_thread,  /* pthread_descr p_nextlive */
   &__pthread_initial_thread,  /* pthread_descr p_prevlive */
   NULL,                       /* pthread_descr p_nextwaiting */
-  0,                          /* pthread_t p_tid */
+  PTHREAD_THREADS_MAX,        /* pthread_t p_tid */
   0,                          /* int p_pid */
   0,                          /* int p_priority */
   &__pthread_handles[0].h_spinlock, /* int * p_spinlock */
@@ -52,7 +52,8 @@ struct _pthread_descr_struct __pthread_initial_thread = {
   0,                          /* int p_errno */
   0,                          /* int p_h_errno */
   PTHREAD_START_ARGS_INITIALIZER, /* struct pthread_start_args p_start_args */
-  {NULL}                      /* void * p_specific[PTHREAD_KEYS_MAX] */
+  {NULL},                     /* void ** p_specific[PTHREAD_KEY_1STLEVEL] */
+  {NULL}                      /* void * p_libc_specific[_LIBC_TSD_KEY_N] */
 };
 
 /* Descriptor of the manager thread; none of this is used but the error
@@ -82,7 +83,8 @@ struct _pthread_descr_struct __pthread_manager_thread = {
   0,                          /* int p_errno */
   0,                          /* int p_h_errno */
   PTHREAD_START_ARGS_INITIALIZER, /* struct pthread_start_args p_start_args */
-  {NULL}                      /* void * p_specific[PTHREAD_KEYS_MAX] */
+  {NULL},                     /* void ** p_specific[PTHREAD_KEYS_MAX] */
+  {NULL}                      /* void * p_libc_specific[_LIBC_TSD_KEY_N] */
 };
 
 /* Pointer to the main thread (the father of the thread manager thread) */
@@ -130,23 +132,22 @@ static void pthread_handle_sigcancel(int sig);
       mask with e.g. sigsetjmp before creating the first thread);
    - a regular function called from pthread_create when needed. */
 
-void __pthread_initialize(void) __attribute__((constructor));
+static void pthread_initialize(void) __attribute__((constructor));
 
-void __pthread_initialize(void)
+static void pthread_initialize(void)
 {
   struct sigaction sa;
   sigset_t mask;
 
   /* If already done (e.g. by a constructor called earlier!), bail out */
-  if (__pthread_initial_thread_bos != NULL)
-    return;
+  if (__pthread_initial_thread_bos != NULL) return;
   /* For the initial stack, reserve at least STACK_SIZE bytes of stack
      below the current stack address, and align that on a
      STACK_SIZE boundary. */
   __pthread_initial_thread_bos =
     (char *)(((long)CURRENT_STACK_FRAME - 2 * STACK_SIZE) & ~(STACK_SIZE - 1));
   /* Update the descriptor for the initial thread. */
-  __pthread_initial_thread.p_pid = getpid();
+  __pthread_initial_thread.p_pid = __getpid();
   /* If we have special thread_self processing, initialize that for the
      main thread now.  */
 #ifdef INIT_THREAD_SELF
@@ -180,7 +181,7 @@ static int pthread_initialize_manager(void)
 
   /* If basic initialization not done yet (e.g. we're called from a
      constructor run before our constructor), do it now */
-  if (__pthread_initial_thread_bos == NULL) __pthread_initialize();
+  if (__pthread_initial_thread_bos == NULL) pthread_initialize();
   /* Setup stack for thread manager */
   __pthread_manager_thread_bos = malloc(THREAD_MANAGER_STACK_SIZE);
   if (__pthread_manager_thread_bos == NULL) return -1;
@@ -361,15 +362,17 @@ void __pthread_reset_main_thread()
 {
   pthread_descr self = thread_self();
 
-  /* Free the thread manager stack */
-  free(__pthread_manager_thread_bos);
-  __pthread_manager_thread_bos = __pthread_manager_thread_tos = NULL;
-  /* Close the two ends of the pipe */
-  __libc_close(__pthread_manager_request);
-  __libc_close(__pthread_manager_reader);
-  __pthread_manager_request = __pthread_manager_reader = -1;
+  if (__pthread_manager_request != -1) {
+    /* Free the thread manager stack */
+    free(__pthread_manager_thread_bos);
+    __pthread_manager_thread_bos = __pthread_manager_thread_tos = NULL;
+    /* Close the two ends of the pipe */
+    __libc_close(__pthread_manager_request);
+    __libc_close(__pthread_manager_reader);
+    __pthread_manager_request = __pthread_manager_reader = -1;
+  }
   /* Update the pid of the main thread */
-  self->p_pid = getpid();
+  self->p_pid = __getpid();
   /* Make the forked thread the main thread */
   __pthread_main_thread = self;
   self->p_nextlive = self;
@@ -397,7 +400,7 @@ void __pthread_message(char * fmt, long arg)
 {
   char buffer[1024];
   va_list args;
-  sprintf(buffer, "%05d : ", getpid());
+  sprintf(buffer, "%05d : ", __getpid());
   va_start(args, fmt);
   vsnprintf(buffer + 8, sizeof(buffer) - 8, fmt, args);
   va_end(args);
