@@ -24,6 +24,8 @@
 /* Get the semaphore structure definition.  */
 #include <semaphorebits.h>
 
+__BEGIN_DECLS
+
 extern int sem_init __P((sem_t *__sem, int __pshared, unsigned int __value));
 extern int sem_destroy __P((sem_t *__sem));
 extern int sem_wait __P((sem_t *__sem));
@@ -31,4 +33,6 @@ extern int sem_trywait __P((sem_t *__sem));
 extern int sem_post __P((sem_t *__sem));
 extern int sem_getvalue __P((sem_t *__sem, int *__sval));
 
+__END_DECLS
+
 #endif  /* semaphore.h */
