@@ -53,7 +53,7 @@ __compare_and_swap (long int *p, long int oldval, long int newval)
   char ret;
   long int readval;
 
-  __asm__ __volatile__ ("lock; cmpxchg %3, %1; sete %0"
+  __asm__ __volatile__ ("lock; cmpxchgl %3, %1; sete %0"
 			: "=q" (ret), "=m" (*p), "=a" (readval)
 			: "r" (newval), "m" (*p), "a" (oldval));
   return ret;
