@@ -1,4 +1,4 @@
-/* Copyright (C) 1996 Free Software Foundation, Inc.
+/* Copyright (C) 1996, 1997 Free Software Foundation, Inc.
    This file is part of the GNU C Library.
 
    The GNU C Library is free software; you can redistribute it and/or
@@ -16,9 +16,10 @@
    write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
    Boston, MA 02111-1307, USA.  */
 
-/* Due to the implementation of the load-locked/store-conditional
-   instructions, we cannot pack semaphores closer than a cache line
-   or risk threads deadlocking on unrelated semaphores.  */
+#ifndef _SEMAPHORE_H
+#error "Never use <semaphorebits.h> directly; include <semaphore.h> instead."
+#endif
+
 
 typedef struct {
   long int sem_status;
